package com.usky.common.security.interceptor;

import com.usky.common.core.constants.SecurityConstants;
import com.usky.common.core.context.SecurityContextHolder;
import com.usky.common.core.util.IpUtils;
import com.usky.common.core.util.JwtUtils;
import com.usky.common.core.util.ServletUtils;
import com.usky.common.core.util.StringUtils;
import com.usky.common.security.auth.AuthUtil;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.system.model.LoginUser;
import io.jsonwebtoken.Claims;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.AsyncHandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Objects;

/**
 * 自定义请求头拦截器，将Header数据封装到线程变量中方便获取
 * 注意：此拦截器会同时验证当前用户有效期自动刷新有效期
 *
 * @author ruoyi
 */
@Slf4j
public class HeaderInterceptor implements AsyncHandlerInterceptor
{
    public static final String REAL_IP_KEY = "X-Real-IP";

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception
    {
        if (!(handler instanceof HandlerMethod))
        {
            return true;
        }
        long startTime = System.currentTimeMillis();
        String remoteHost = request.getRemoteHost();
        String realIP = request.getHeader(REAL_IP_KEY);
        log.info("access {} form realIP[{}] with proxyIP[{}] ",request.getRequestURI(),realIP==null?remoteHost:realIP,remoteHost);
        SecurityContextHolder.setUserId(ServletUtils.getHeader(request, SecurityConstants.DETAILS_USER_ID));
        SecurityContextHolder.setUserName(ServletUtils.getHeader(request, SecurityConstants.DETAILS_USERNAME));
        SecurityContextHolder.setUserKey(ServletUtils.getHeader(request, SecurityConstants.USER_KEY));
        SecurityContextHolder.setTenantId(ServletUtils.getHeader(request, SecurityConstants.DETAILS_TENANT_ID));
        SecurityContextHolder.set("startTime",String.valueOf(startTime));
        String token = SecurityUtils.getToken();
        if (StringUtils.isNotEmpty(token))
        {
            Claims claims = JwtUtils.parseToken(token);
            String userKey = JwtUtils.getUserKey(claims);
            String userId = JwtUtils.getUserId(claims);
            String userName = JwtUtils.getUserName(claims);
            String tenantId = JwtUtils.getTenantId(claims);

            SecurityContextHolder.setUserId(userId);
            SecurityContextHolder.setUserName(userName);
            SecurityContextHolder.setUserKey(userKey);
            SecurityContextHolder.setTenantId(tenantId);

            log.info("access session user the userId {} and userName {} and tenantId {}",userId,userName,tenantId);
            //TODO 2022-05-03 gez 每次请求都要从rdis获取用户信息，并刷新时间，这个代码拉低每次请求效率
            // 如果不影响其它业务模块，考虑去掉这块逻辑
            LoginUser loginUser = AuthUtil.getLoginUser(token);
            if (Objects.nonNull(loginUser))
            {
                AuthUtil.verifyLoginUserExpire(loginUser);
                SecurityContextHolder.set(SecurityConstants.LOGIN_USER, loginUser);
            }
        }
        return true;
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) throws Exception
    {
        long endTime = System.currentTimeMillis();
        String startTime = SecurityContextHolder.get("startTime");
        if(StringUtils.isNotEmpty(startTime)) {
            long startTimeOfLong = Long.valueOf(startTime);
            long l = endTime - startTimeOfLong;
            log.info("access {} finish cost [{}] ms", request.getRequestURI(), l);
        }
        SecurityContextHolder.remove();
    }
}
